@@ -50,6 +50,9 @@
 
 #include <vector>
 
+#include "opencv2/core/core.hpp"
+#include "opencv2/core/types_c.h"
+
 #if defined WIN32 || defined _WIN32
 #  ifndef WIN32
 #    define WIN32
@@ -59,36 +62,27 @@
 #  endif
 #endif
 
-#if defined WIN32 || defined WINCE
-#ifndef _WIN32_WINNT         // This is needed for the declaration of TryEnterCriticalSection in winbase.h with Visual Studio 2005 (and older?)
-#define _WIN32_WINNT 0x0400  // http://msdn.microsoft.com/en-us/library/ms686857(VS.85).aspx
-#endif
-#include <windows.h>
-#undef small
-#undef min
-#undef max
-#else
-#include <pthread.h>
-#include <sys/mman.h>
+#if !defined WIN32 && !defined WINCE
+#  include <pthread.h>
 #endif
 
 #ifdef __BORLANDC__
-#ifndef WIN32
-    #define     WIN32
-#endif
-#ifndef _WIN32
-    #define     _WIN32
-#endif
-    #define     CV_DLL
-    #undef      _CV_ALWAYS_PROFILE_
-    #define     _CV_ALWAYS_NO_PROFILE_
+#  ifndef WIN32
+#    define WIN32
+#  endif
+#  ifndef _WIN32
+#    define _WIN32
+#  endif
+#  define CV_DLL
+#  undef _CV_ALWAYS_PROFILE_
+#  define _CV_ALWAYS_NO_PROFILE_
 #endif
 
 #ifndef FALSE
-#define FALSE 0
+#  define FALSE 0
 #endif
 #ifndef TRUE
-#define TRUE 1
+#  define TRUE 1
 #endif
 
 #define __BEGIN__ __CV_BEGIN__
@@ -96,7 +90,7 @@
 #define EXIT __CV_EXIT__
 
 #ifdef HAVE_IPP
-#include "ipp.h"
+#  include "ipp.h"
 
 CV_INLINE IppiSize ippiSize(int width, int height)
 {
@@ -105,110 +99,195 @@ CV_INLINE IppiSize ippiSize(int width, int height)
 }
 #endif
 
-#if defined __SSE2__ || _MSC_VER >= 1300
-#include "emmintrin.h"
-#define CV_SSE 1
-#define CV_SSE2 1
-#if defined __SSE3__ || _MSC_VER >= 1500
-#include "pmmintrin.h"
-#define CV_SSE3 1
-#endif
-#else
-#define CV_SSE 0
-#define CV_SSE2 0
-#define CV_SSE3 0
+#ifndef IPPI_CALL
+#  define IPPI_CALL(func) CV_Assert((func) >= 0)
 #endif
 
-#ifndef IPPI_CALL
-#define IPPI_CALL(func) CV_Assert((func) >= 0)
+#if defined __SSE2__ || defined _M_X64  || (defined _M_IX86_FP && _M_IX86_FP >= 2)
+#  include "emmintrin.h"
+#  define CV_SSE 1
+#  define CV_SSE2 1
+#  if defined __SSE3__ || (defined _MSC_VER && _MSC_VER >= 1500)
+#    include "pmmintrin.h"
+#    define CV_SSE3 1
+#  endif
+#  if defined __SSSE3__  || (defined _MSC_VER && _MSC_VER >= 1500)
+#    include "tmmintrin.h"
+#    define CV_SSSE3 1
+#  endif
+#  if defined __SSE4_1__ || (defined _MSC_VER && _MSC_VER >= 1500)
+#    include <smmintrin.h>
+#    define CV_SSE4_1 1
+#  endif
+#  if defined __SSE4_2__ || (defined _MSC_VER && _MSC_VER >= 1500)
+#    include <nmmintrin.h>
+#    define CV_SSE4_2 1
+#  endif
+#  if defined __AVX__ || (defined _MSC_FULL_VER && _MSC_FULL_VER >= 160040219)
+// MS Visual Studio 2010 (2012?) has no macro pre-defined to identify the use of /arch:AVX
+// See: http://connect.microsoft.com/VisualStudio/feedback/details/605858/arch-avx-should-define-a-predefined-macro-in-x64-and-set-a-unique-value-for-m-ix86-fp-in-win32
+#    include <immintrin.h>
+#    define CV_AVX 1
+#    if defined(_XCR_XFEATURE_ENABLED_MASK)
+#      define __xgetbv() _xgetbv(_XCR_XFEATURE_ENABLED_MASK)
+#    else
+#      define __xgetbv() 0
+#    endif
+#  endif
+#endif
+
+#ifdef __ARM_NEON__
+#  include <arm_neon.h>
+#  define CV_NEON 1
+#  define CPU_HAS_NEON_FEATURE (true)
+#endif
+
+#ifndef CV_SSE
+#  define CV_SSE 0
+#endif
+#ifndef CV_SSE2
+#  define CV_SSE2 0
+#endif
+#ifndef CV_SSE3
+#  define CV_SSE3 0
+#endif
+#ifndef CV_SSSE3
+#  define CV_SSSE3 0
+#endif
+#ifndef CV_SSE4_1
+#  define CV_SSE4_1 0
+#endif
+#ifndef CV_SSE4_2
+#  define CV_SSE4_2 0
+#endif
+#ifndef CV_AVX
+#  define CV_AVX 0
+#endif
+#ifndef CV_NEON
+#  define CV_NEON 0
 #endif
 
 #ifdef HAVE_TBB
-    #include "tbb/tbb_stddef.h"
-    #if TBB_VERSION_MAJOR*100 + TBB_VERSION_MINOR >= 202
-        #include "tbb/tbb.h"
-        #undef min
-        #undef max
-    #else
-        #undef HAVE_TBB
-    #endif
+#  include "tbb/tbb_stddef.h"
+#  if TBB_VERSION_MAJOR*100 + TBB_VERSION_MINOR >= 202
+#    include "tbb/tbb.h"
+#    include "tbb/task.h"
+#    undef min
+#    undef max
+#  else
+#    undef HAVE_TBB
+#  endif
 #endif
 
-#ifdef HAVE_EIGEN2
-    #include <Eigen/Core>
-    #include "opencv2/core/eigen.hpp"
+#ifdef HAVE_EIGEN
+#  if defined __GNUC__ && defined __APPLE__
+#    pragma GCC diagnostic ignored "-Wshadow"
+#  endif
+#  include <Eigen/Core>
+#  include "opencv2/core/eigen.hpp"
 #endif
 
 #ifdef __cplusplus
 
+namespace cv
+{
 #ifdef HAVE_TBB
-    namespace cv
-    {
-        typedef tbb::blocked_range<int> BlockedRange;
-        
-        template<typename Body> static inline
-        void parallel_for( const BlockedRange& range, const Body& body )
-        {
-            tbb::parallel_for(range, body);
-        }
-        
-        template<typename Iterator, typename Body> static inline
-        void parallel_do( Iterator first, Iterator last, const Body& body )
-        {
-            tbb::parallel_do(first, last, body);
-        }
-        
-        typedef tbb::split Split;
-        
-        template<typename Body> static inline
-        void parallel_reduce( const BlockedRange& range, Body& body )
-        {
-            tbb::parallel_reduce(range, body);
-        }
-        
-        typedef tbb::concurrent_vector<Rect> ConcurrentRectVector;
-    }
-#else
-    namespace cv
-    {
-        class BlockedRange
-        {
-        public:
-            BlockedRange() : _begin(0), _end(0), _grainsize(0) {}
-            BlockedRange(int b, int e, int g=1) : _begin(b), _end(e), _grainsize(g) {}
-            int begin() const { return _begin; }
-            int end() const { return _end; }
-            int grainsize() const { return _grainsize; }
-            
-        protected:
-            int _begin, _end, _grainsize;
-        };
 
-        template<typename Body> static inline
-        void parallel_for( const BlockedRange& range, const Body& body )
-        {
-            body(range);
-        }
-        
-        template<typename Iterator, typename Body> static inline
-        void parallel_do( Iterator first, Iterator last, const Body& body )
-        {
-            for( ; first != last; ++first )
-                body(*first);
-        }
-        
-        class Split {};
-        
-        template<typename Body> static inline
-        void parallel_reduce( const BlockedRange& range, Body& body )
-        {
-            body(range);
-        }
-        
-        typedef std::vector<Rect> ConcurrentRectVector;
+    typedef tbb::blocked_range<int> BlockedRange;
+
+    template<typename Body> static inline
+    void parallel_for( const BlockedRange& range, const Body& body )
+    {
+        tbb::parallel_for(range, body);
+    }
+
+    template<typename Iterator, typename Body> static inline
+    void parallel_do( Iterator first, Iterator last, const Body& body )
+    {
+        tbb::parallel_do(first, last, body);
+    }
+
+    typedef tbb::split Split;
+
+    template<typename Body> static inline
+    void parallel_reduce( const BlockedRange& range, Body& body )
+    {
+        tbb::parallel_reduce(range, body);
+    }
+
+    typedef tbb::concurrent_vector<Rect> ConcurrentRectVector;
+    typedef tbb::concurrent_vector<double> ConcurrentDoubleVector;
+#else
+    class BlockedRange
+    {
+    public:
+        BlockedRange() : _begin(0), _end(0), _grainsize(0) {}
+        BlockedRange(int b, int e, int g=1) : _begin(b), _end(e), _grainsize(g) {}
+        int begin() const { return _begin; }
+        int end() const { return _end; }
+        int grainsize() const { return _grainsize; }
+
+    protected:
+        int _begin, _end, _grainsize;
+    };
+
+    template<typename Body> static inline
+    void parallel_for( const BlockedRange& range, const Body& body )
+    {
+        body(range);
+    }
+    typedef std::vector<Rect> ConcurrentRectVector;
+    typedef std::vector<double> ConcurrentDoubleVector;
+
+    template<typename Iterator, typename Body> static inline
+    void parallel_do( Iterator first, Iterator last, const Body& body )
+    {
+        for( ; first != last; ++first )
+            body(*first);
+    }
+
+    class Split {};
+
+    template<typename Body> static inline
+    void parallel_reduce( const BlockedRange& range, Body& body )
+    {
+        body(range);
     }
 #endif
-#endif
+
+    // Returns a static string if there is a parallel framework,
+    // NULL otherwise.
+    CV_EXPORTS const char* currentParallelFramework();
+} //namespace cv
+
+#define CV_INIT_ALGORITHM(classname, algname, memberinit) \
+    static ::cv::Algorithm* create##classname() \
+    { \
+        return new classname; \
+    } \
+    \
+    static ::cv::AlgorithmInfo& classname##_info() \
+    { \
+        static ::cv::AlgorithmInfo classname##_info_var(algname, create##classname); \
+        return classname##_info_var; \
+    } \
+    \
+    static ::cv::AlgorithmInfo& classname##_info_auto = classname##_info(); \
+    \
+    ::cv::AlgorithmInfo* classname::info() const \
+    { \
+        static volatile bool initialized = false; \
+        \
+        if( !initialized ) \
+        { \
+            initialized = true; \
+            classname obj; \
+            memberinit; \
+        } \
+        return &classname##_info(); \
+    }
+
+#endif //__cplusplus
 
 /* maximal size of vector to run matrix operations on it inline (i.e. w/o ipp calls) */
 #define  CV_MAX_INLINE_MAT_OP_SIZE  10
@@ -252,9 +331,9 @@ CV_INLINE IppiSize ippiSize(int width, int height)
 #define  CV_MAX_STRLEN  1024
 
 #if 0 /*def  CV_CHECK_FOR_NANS*/
-    #define CV_CHECK_NANS( arr ) cvCheckArray((arr))
+#  define CV_CHECK_NANS( arr ) cvCheckArray((arr))
 #else
-    #define CV_CHECK_NANS( arr )
+#  define CV_CHECK_NANS( arr )
 #endif
 
 /****************************************************************************************\
@@ -263,40 +342,38 @@ CV_INLINE IppiSize ippiSize(int width, int height)
 
 /* get alloca declaration */
 #ifdef __GNUC__
-    #undef alloca
-    #define alloca __builtin_alloca
+#  undef alloca
+#  define alloca __builtin_alloca
+#  define CV_HAVE_ALLOCA 1
 #elif defined WIN32 || defined _WIN32 || \
       defined WINCE || defined _MSC_VER || defined __BORLANDC__
-    #include <malloc.h>
+#  include <malloc.h>
+#  define CV_HAVE_ALLOCA 1
 #elif defined HAVE_ALLOCA_H
-    #include <alloca.h>
+#  include <alloca.h>
+#  define CV_HAVE_ALLOCA 1
 #elif defined HAVE_ALLOCA
-    #include <stdlib.h>
+#  include <stdlib.h>
+#  define CV_HAVE_ALLOCA 1
 #else
-    #error "No alloca!"
+#  undef CV_HAVE_ALLOCA
 #endif
 
 #ifdef __GNUC__
-#define CV_DECL_ALIGNED(x) __attribute__ ((aligned (x)))
+#  define CV_DECL_ALIGNED(x) __attribute__ ((aligned (x)))
 #elif defined _MSC_VER
-#define CV_DECL_ALIGNED(x) __declspec(align(x))
+#  define CV_DECL_ALIGNED(x) __declspec(align(x))
 #else
-#define CV_DECL_ALIGNED(x)
+#  define CV_DECL_ALIGNED(x)
 #endif
 
+#if CV_HAVE_ALLOCA
 /* ! DO NOT make it an inline function */
-#define cvStackAlloc(size) cvAlignPtr( alloca((size) + CV_MALLOC_ALIGN), CV_MALLOC_ALIGN )
-
-#if defined _MSC_VER || defined __BORLANDC__
-    #define CV_BIG_INT(n)   n##I64
-    #define CV_BIG_UINT(n)  n##UI64
-#else
-    #define CV_BIG_INT(n)   n##LL
-    #define CV_BIG_UINT(n)  n##ULL
+#  define cvStackAlloc(size) cvAlignPtr( alloca((size) + CV_MALLOC_ALIGN), CV_MALLOC_ALIGN )
 #endif
 
 #ifndef CV_IMPL
-#define CV_IMPL CV_EXTERN_C
+#  define CV_IMPL CV_EXTERN_C
 #endif
 
 #define CV_DBG_BREAK() { volatile int* crashMe = 0; *crashMe = 0; }
@@ -636,25 +713,25 @@ typedef enum CvStatus
     CV_UNSUPPORTED_DEPTH_ERR    = -101,
     CV_UNSUPPORTED_FORMAT_ERR   = -100,
 
-    CV_BADARG_ERR      = -49,  //ipp comp
-    CV_NOTDEFINED_ERR  = -48,  //ipp comp
+    CV_BADARG_ERR               = -49,  //ipp comp
+    CV_NOTDEFINED_ERR           = -48,  //ipp comp
 
-    CV_BADCHANNELS_ERR = -47,  //ipp comp
-    CV_BADRANGE_ERR    = -44,  //ipp comp
-    CV_BADSTEP_ERR     = -29,  //ipp comp
+    CV_BADCHANNELS_ERR          = -47,  //ipp comp
+    CV_BADRANGE_ERR             = -44,  //ipp comp
+    CV_BADSTEP_ERR              = -29,  //ipp comp
 
-    CV_BADFLAG_ERR     =  -12,
-    CV_DIV_BY_ZERO_ERR =  -11, //ipp comp
-    CV_BADCOEF_ERR     =  -10,
+    CV_BADFLAG_ERR              =  -12,
+    CV_DIV_BY_ZERO_ERR          =  -11, //ipp comp
+    CV_BADCOEF_ERR              =  -10,
 
-    CV_BADFACTOR_ERR   =  -7,
-    CV_BADPOINT_ERR    =  -6,
-    CV_BADSCALE_ERR    =  -4,
-    CV_OUTOFMEM_ERR    =  -3,
-    CV_NULLPTR_ERR     =  -2,
-    CV_BADSIZE_ERR     =  -1,
-    CV_NO_ERR          =   0,
-    CV_OK              =   CV_NO_ERR
+    CV_BADFACTOR_ERR            =  -7,
+    CV_BADPOINT_ERR             =  -6,
+    CV_BADSCALE_ERR             =  -4,
+    CV_OUTOFMEM_ERR             =  -3,
+    CV_NULLPTR_ERR              =  -2,
+    CV_BADSIZE_ERR              =  -1,
+    CV_NO_ERR                   =   0,
+    CV_OK                       =   CV_NO_ERR
 }
 CvStatus;
 
@@ -669,8 +746,7 @@ CvFuncTable;
 typedef struct CvBigFuncTable
 {
     void*   fn_2d[CV_DEPTH_MAX*4];
-}
-CvBigFuncTable;
+} CvBigFuncTable;
 
 #define CV_INIT_FUNC_TAB( tab, FUNCNAME, FLAG )         \
     (tab).fn_2d[CV_8U] = (void*)FUNCNAME##_8u##FLAG;    \
@@ -681,4 +757,47 @@ CvBigFuncTable;
     (tab).fn_2d[CV_32F] = (void*)FUNCNAME##_32f##FLAG;  \
     (tab).fn_2d[CV_64F] = (void*)FUNCNAME##_64f##FLAG
 
+#ifdef __cplusplus
+
+// < Deprecated
+
+class CV_EXPORTS CvOpenGlFuncTab
+{
+public:
+    virtual ~CvOpenGlFuncTab();
+
+    virtual void genBuffers(int n, unsigned int* buffers) const = 0;
+    virtual void deleteBuffers(int n, const unsigned int* buffers) const = 0;
+
+    virtual void bufferData(unsigned int target, ptrdiff_t size, const void* data, unsigned int usage) const = 0;
+    virtual void bufferSubData(unsigned int target, ptrdiff_t offset, ptrdiff_t size, const void* data) const = 0;
+
+    virtual void bindBuffer(unsigned int target, unsigned int buffer) const = 0;
+
+    virtual void* mapBuffer(unsigned int target, unsigned int access) const = 0;
+    virtual void unmapBuffer(unsigned int target) const = 0;
+
+    virtual void generateBitmapFont(const std::string& family, int height, int weight, bool italic, bool underline, int start, int count, int base) const = 0;
+
+    virtual bool isGlContextInitialized() const = 0;
+};
+
+CV_EXPORTS void icvSetOpenGlFuncTab(const CvOpenGlFuncTab* tab);
+
+CV_EXPORTS bool icvCheckGlError(const char* file, const int line, const char* func = "");
+
+// >
+
+namespace cv { namespace ogl {
+CV_EXPORTS bool checkError(const char* file, const int line, const char* func = "");
+}}
+
+#if defined(__GNUC__)
+    #define CV_CheckGlError() CV_DbgAssert( (cv::ogl::checkError(__FILE__, __LINE__, __func__)) )
+#else
+    #define CV_CheckGlError() CV_DbgAssert( (cv::ogl::checkError(__FILE__, __LINE__)) )
 #endif
+
+#endif //__cplusplus
+
+#endif // __OPENCV_CORE_INTERNAL_HPP__
